#include <glib.h>
#include <glib-object.h>
#include "makuobject.h"
#include "marshallers.h"


static void maku_object_class_init (gpointer g_class, gpointer g_class_data);
static void maku_object_init (GTypeInstance *instance, gpointer g_class);
static GObject * maku_object_constructor (GType type, guint n_construct_prop, GObjectConstructParam *construct_prop);
static void maku_object_finalize (GObject *object);
static DBusGProxy * get_dbus_gproxy (MakuObject *self);
void name_owner_changed_callback (DBusGProxy *gproxy, gchar *arg1, gchar *arg2, gchar *arg3, gpointer user_data);
void session_idle_changed_callback (DBusGProxy *gproxy, gboolean arg, gpointer user_data);


static gpointer maku_object_parent_class = NULL;
static guint maku_object_signal [LAST_SIGNAL] = {0};


/* Use dbus-monitor or dbus-viewer to dind these values easily */
#define DBUS_NAME_SCREEN		"org.gnome.ScreenSaver"
#define DBUS_PATH_SCREEN		"/org/gnome/ScreenSaver"
#define DBUS_INTERFACE_SCREEN		"org.gnome.ScreenSaver"


// returns the type
GType maku_object_get_type (void)
{
	static GType type = 0;
	
	if (G_UNLIKELY (type == 0))
	{
		static const GTypeInfo info = 
		{
			sizeof (MakuObjectClass),
			NULL,
			NULL,
			(GClassInitFunc) maku_object_class_init,
			NULL,
			NULL,
			sizeof (MakuObject),
			0,
			(GInstanceInitFunc) maku_object_init,
			NULL
		};
		
		type = g_type_register_static (G_TYPE_OBJECT, "MakuObject", &info, 0);
	}
	return type;
}



// class init
static void maku_object_class_init (gpointer g_class, gpointer g_class_data)
{
	GObjectClass *gobj_class = G_OBJECT_CLASS (g_class);
	MakuObjectClass *maku_obj_class = MAKU_OBJECT_CLASS (g_class);
	
	maku_object_parent_class = g_type_class_peek_parent (maku_obj_class);
	
	gobj_class->constructor = maku_object_constructor;
	gobj_class->finalize = maku_object_finalize;
	
	/* signals */
	/* The signal will carry the payload originally carried by the DBus signal */			 
	maku_object_signal [SESSION_IDLE_CHANGED] = g_signal_new ("session-idle-changed",\
							MAKU_TYPE_OBJECT,\
							G_SIGNAL_RUN_FIRST,\
							0,\
							NULL,\
							NULL,\
							g_cclosure_marshal_VOID__BOOLEAN,\
							G_TYPE_NONE,\
							1,\
							G_TYPE_BOOLEAN);
	
}



// instance init for MakuObject
static void maku_object_init (GTypeInstance *instance, gpointer g_class)
{
	MakuObject *self = (MakuObject *)instance;
	
	// initialize the DBus proxy
	self->gproxy = get_dbus_gproxy (self);
	
	// other stuff!
	self->age = 100;
}



// constructor to override GObject's constructor.
static GObject * maku_object_constructor (GType type, guint n_construct_prop, GObjectConstructParam *construct_prop)
{
	GObject *gobject = NULL;
	MakuObjectClass *maku_obj_class = NULL;
	GObjectClass *gobj_class = NULL;

	maku_obj_class = MAKU_OBJECT_CLASS (g_type_class_peek (MAKU_TYPE_OBJECT));
	gobj_class = G_OBJECT_CLASS (g_type_class_peek_parent (maku_obj_class));
	
	gobject = gobj_class->constructor (type, n_construct_prop, construct_prop);
	
	return gobject;
}



static void maku_object_finalize (GObject *object)
{
	MakuObject *self = MAKU_OBJECT (object);
	
	g_print ("Finalizing object!\n");
	if (self)
		g_object_unref (self->gproxy);
}



/* Creates a DBusGProxy for the object & connects to any interested signals */
static DBusGProxy * get_dbus_gproxy (MakuObject *self)
{
	GError *error = NULL;
	DBusGProxy *gproxy = NULL;
	DBusGConnection *connection = NULL;
	gboolean test = 0;
	gint i=0;

	connection = dbus_g_bus_get (DBUS_BUS_SESSION, &error);
	if (!connection) {
		g_print ("Failed to get connection");
		g_error_free (error);
		return NULL;
	}

	/* 	
		TODO - Attaching signals should be moved to a new function - maybe a wrapper... 
		If you want to use a custom marshaller, use register_marshaller function as below 
	*/
	/*
	dbus_g_object_register_marshaller (g_cclosure_user_marshal_VOID__STRING_STRING_STRING, G_TYPE_NONE, \
						G_TYPE_STRING, G_TYPE_STRING, G_TYPE_STRING, G_TYPE_INVALID);
	*/
							
	
	/* DBusGProxy for SCREEN_SAVER */
	g_assert ((gproxy = dbus_g_proxy_new_for_name (connection,\
							DBUS_NAME_SCREEN,\
							DBUS_PATH_SCREEN,\
							DBUS_INTERFACE_SCREEN)));	
	
	dbus_g_proxy_add_signal (gproxy, "SessionIdleChanged", G_TYPE_BOOLEAN, G_TYPE_INVALID);
	dbus_g_proxy_connect_signal (gproxy, "SessionIdleChanged", G_CALLBACK (session_idle_changed_callback), self, NULL);	
	
	g_print ("Going in for the kill!\n");
	for (i=0; i<1; i++)
	{
		if (!dbus_g_proxy_call (gproxy, "GetActive", &error, G_TYPE_INVALID, test,\
				G_TYPE_INVALID))
			g_print ("GetSessionIdleTime call could not be completed\n");
		else
			g_print ("GetSessionIdleTime retured time\n");
	}	       
	g_print ("Signals connected...\n");
	
	return gproxy;
}



/* Wrapping  signal SessionPowerManagementIdleChanged */
void session_idle_changed_callback (DBusGProxy *gproxy, gboolean arg, gpointer user_data)
{
	MakuObject *maku_object = MAKU_OBJECT (user_data);
	
	g_print ("Got a screen_saver Signal.\n");
	g_print ("Signal's Args : [%d]\n", arg);
	g_print ("Re-emitting signal as a g-signal!\n");
	g_signal_emit_by_name (maku_object, "session-idle-changed", arg);
}


/* junk method for the object */
void maku_object_say_hello (MakuObject *object)
{
	g_print ("MakuObject just sent you a Hello :), self->gproxy is at %u\n", &object->gproxy);
}

/* junk method for the object */
void maku_object_say_cheese (int i)
{
	gint j;
	for (j=0; j<i; j++)
		g_print ("MakuObject just sent you a CHEESE! :)\n");
}









